/*
 *  Copyright (c) 2008 Luca Abeni
 *
 *  This is free software; see GPL.txt
 */
#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <sys/time.h>
#include <time.h>
#include <netinet/in.h>

#include "rtcp.h"

static uint32_t time_to_ntp(struct timeval *tv)
{
    uint64_t frac;
    uint32_t time;

    frac = tv->tv_usec;
    frac = (frac << 16) / 1000000;
    time = tv->tv_sec + 2208988800u;
    time = time << 16;
    time |= frac;

    return time;
}

static int rtcp_rr_parse(const uint8_t *pkt, struct rtcp_stats *stats)
{
    struct timeval timeval_received;
    unsigned loss_stats;
    double time_since_last_RTCP;

    //printf("SSRC: %d\n", ntohl(*(unsigned *)(pkt + 0)));
    //printf("SSRC1: %d\n", ntohl(*(unsigned *)(pkt + 4)));

    loss_stats = ntohl(*(unsigned *)(pkt + 8));

    stats->lost_frac = loss_stats >> 24;
    stats->lost_num = loss_stats & 0xFFFFFF;
    stats->last_recv = ntohl(*(unsigned *)(pkt + 12));
    stats->jitter = ntohl(*(unsigned *)(pkt + 16));
    stats->LSR = ntohl(*(unsigned *)(pkt + 20));
    stats->DLSR = ntohl(*(unsigned *)(pkt + 24));

    gettimeofday(&timeval_received, NULL);
    if(stats->LSR) {
        stats->RTT = time_to_ntp(&timeval_received) - stats->LSR - stats->DLSR;
    }

    time_since_last_RTCP = (timeval_received.tv_sec - stats->timeval_last_RTCP.tv_sec) +
                                   ((timeval_received.tv_usec - stats->timeval_last_RTCP.tv_usec) / 1000000.0);
    stats->bit_rate = stats->sum * 8 / time_since_last_RTCP;
    stats->avg_pkt_size = stats->sum * 8 / stats->sent_pkts;
    stats->sent_pkts = 0;
    stats->sum = 0;
    stats->timeval_last_RTCP.tv_sec = timeval_received.tv_sec;
    stats->timeval_last_RTCP.tv_usec = timeval_received.tv_usec;

    return 0;
}

int rtcp_parse(const uint8_t *pkt, struct rtcp_stats *stats)
{
    int len;

    if (pkt[0] >> 6 != 2) {
        fprintf(stderr, "RTP Version %d != 2!\n", pkt[0] >> 6);

        return -1;
    }
    /*printf("RC: %d\n", pkt[0] & 0x1F);*/

    len = ntohs(*(unsigned short *)(pkt + 2));
    if (len % 7) {
        fprintf(stderr, "Wrong len %d\n", len);

	return -1;
    }
    switch (pkt[1]) {
        case 201:
            return rtcp_rr_parse(pkt + 4, stats);
        default:
            fprintf(stderr, "Unsupported RTCP type %d\n", pkt[1]);
    }

    return 0;
}

struct rtcp_stats *rtcp_new(void)
{
    struct rtcp_stats *r;

    r = malloc(sizeof(struct rtcp_stats));
    if (r == NULL) {
        return NULL;
    }
    memset(r, 0, sizeof(struct rtcp_stats));

    return r;
}
