/*
 *  Copyright (c) 2008 Luca Abeni
 *
 *  This is free software; see GPL.txt
 */
#include <stdarg.h>
#include <unistd.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "rtp.h"
#include "rtcp.h"
#include "session.h"
#include "in.h"
#include "sdp.h"

#define MAX_OUT_MEDIA 10
struct session {
    struct insession *in_session;
    struct session *next;
    int id;

    struct rtp_session *rtp[MAX_OUT_MEDIA];
    int tracks;
};

static struct session *first_session;
static int global_session_id;

struct insession *insession_from_id(int id)
{
    struct session *s;

    s = first_session;
    while (s != NULL) {
        if (s->id == id) {
            return s->in_session;
	}

        s = s->next;
    }

    return NULL;
}

static void *session_from_id(int session_id)
{
    struct session *s;

    s = first_session;
    while ((s != NULL) && (s->id != session_id)) {
        s = s->next;
    }

    return s;
}

void session_play(int session_id)
{
    struct session *os;
    int i;

    os = session_from_id(session_id);
    if (os == NULL) {
        fprintf(stderr, "This cannot happen!\n");
        abort();
    }

    for (i = 0; i < MAX_OUT_MEDIA; i++) {
        if (os->rtp[i]) {
            insession_play(os->in_session, os->rtp[i], i);
        }
    }
}

void session_stop(int session_id)
{
    struct session *os;
    int i;

    os = session_from_id(session_id);
    if (os == NULL) {
        fprintf(stderr, "This cannot happen!\n");
        abort();
    }

    for (i = 0; i < MAX_OUT_MEDIA; i++) {
        if (os->rtp[i]) {
            insession_stop(os->in_session, os->rtp[i]);
rtp_session_close(os->rtp[i]);	/* FIXME: Check this!!! */
        }
    }
    insession_release(os->in_session);
}

int session_new(void *s)
{
    struct session *os;

    os = malloc(sizeof(struct session));
    if (os == NULL) {
        return -1;
    }
    memset(os, 0, sizeof(struct session));

    os->in_session = s;
    os->next = first_session;
    os->id = ++global_session_id;
    first_session = os;

    return os->id;
}

int session_delete(int id)
{
    struct session *s;

    if (first_session && (first_session->id == id)) {
        s = first_session;
        first_session = first_session->next;
        free(s);

        return 1;
    }
    s = first_session;
    while (s != NULL) {
        if (s->next && (s->next->id == id)) {
            struct session *s1 = s->next;

            s->next = s->next->next;
            free(s1);
            return 1;
	}

        s = s->next;
    }

    return -1;
}

int session_add_stream(int id, struct rtp_session *r, int tid)
{
    struct session *s;

    s = first_session;
    while (s != NULL) {
        if (s->id == id) {
            if (insession_media(s->in_session, tid)->type == MEDIA_TYPE_VIDEO) {
                r->session_id = id;
            } else {
                r->session_id = -1;
            }
            s->rtp[tid] = r;
            return 1;
	}

        s = s->next;
    }

    return -1;
}

int session_bitrate_adjust(int session_id, int bitrate)
{
    int i, video_stream, new_stream, new_bitrate;
    struct insession *s;
    struct session *os;

    os = session_from_id(session_id);
    s = os->in_session;

    video_stream = -1;
    new_stream = 0;
    new_bitrate = 0;
    for (i = 0; i < MAX_MEDIA; i++) {
        if (insession_media(s, i) && (insession_media(s, i)->type == MEDIA_TYPE_VIDEO)) {
            int media_bitrate;

	    if (video_stream == -1) {
                video_stream = i;
            }
            media_bitrate = insession_media(s, i)->bitrate;
	    if (((media_bitrate > bitrate) && (media_bitrate < new_bitrate)) || ((media_bitrate < bitrate) && (media_bitrate > new_bitrate))){
               new_bitrate = media_bitrate;
               new_stream = i;
            }
        }
    }

    insession_stop(s, os->rtp[video_stream]);
    insession_play(s, os->rtp[video_stream], new_stream);

    return new_bitrate;
}
