/*
 *  Copyright (c) 2008 Luca Abeni
 *
 *  This is free software; see GPL.txt
 */
#include <stdint.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "net.h"
#include "engine.h"
#include "rtcp.h"
#include "stats.h"

static const char *output_path;
static int stats_handle(int s, void *c)
{
    int fd, client_ip;
    static char *out;

    out = strdup("Stats\n");

    fd = new_connection(s, &client_ip);
    write(fd, out, strlen(out));
    free(out);
    close(fd);

    return 0;
}

void tlog_init(void *h, int port)
{
    int s, res;

    fprintf(stderr, "Opening %s %d\n", "127.0.0.1", port);
    s = server_init("127.0.0.1", port);
    if (s < 0) {
        return;
    }
    res = event_new(h, s, stats_handle, h);
    if (res < 0) {
        fprintf(stderr, "Cannot create statistics event!");
        close(s);

        return;
    }
}

void stats_file_init(const char* fname)
{
    output_path = strdup(fname);
}

void init_rtcp_stats(void *c, const char *dst, int port)
{
    if (output_path) {
        struct rtcp_stats *r;
        char fname[256];

        sprintf(fname, "%s/%p", output_path, c);
        r = c;
        r->stats_file = fopen(fname, "w");

        if(r->stats_file == NULL){
            fprintf(stderr, "Error: cannot open file %s\n", fname);

            return;
        }

        fprintf(r->stats_file, "#Stream %p starts (to %s/%d)\n", c, dst, port);
        fprintf(r->stats_file, "#Session ID   Lost Frac      Jitter         RTT     Bitrate    Pkt Size\n");

        fflush(r->stats_file);
    }
}

void cleanup_rtcp_stats(void *c)
{
    if (c != NULL) {
        struct rtcp_stats *r;

        r = c;
	if (r->stats_file) {
            fprintf(r->stats_file, "#Stream closed\n");
            fclose(r->stats_file);
        }
        free(r);
    }
}

int print_rtcp_stats(uint64_t t, void *c)
{
    struct rtcp_stats *r;
    const double b = 2;
    double th;

    r=c;

    if(r->stats_file == NULL){
        return -1;
    }

    fprintf(r->stats_file, "%11p  ", r);
    fprintf(r->stats_file, "%10u  ", r->lost_frac);
    fprintf(r->stats_file, "%10u  ", r->jitter);
    fprintf(r->stats_file, "%10f  ", r->RTT / 65536.0);

/* Throughput from TFRC (RFC 3448):
                                   s
   X =  ----------------------------------------------------------
        R*sqrt(2*b*p/3) + (t_RTO * (3*sqrt(3*b*p/8) * p * (1+32*p^2)))
*/
   if (r->lost_frac == 0) {
       th = r->bit_rate / 8;
   } else {
       th = (r->avg_pkt_size / 8) /
              ((r->RTT / 65536.0) * sqrt(2 * b * (r->lost_frac / 256.0) / 3) +
               (4 * (r->RTT / 65536.0) * (3 * sqrt(3 * b * (r->lost_frac / 256.0) / 8)) *
                (r->lost_frac / 256.0) * (1 + 32 * (r->lost_frac / 256.0) * (r->lost_frac / 256.0))));
   }

    /*fprintf(stderr, "Th: %f / %f\n", th, r->bit_rate / 8);*/
    fprintf(r->stats_file, "%5d%5d", (int)(r->bit_rate / 1000), (int)((th * 8) / 1000));
    fprintf(r->stats_file, "%10d\n", r->avg_pkt_size);

    fflush(r->stats_file);

    return 0;
}
